from kuto.core import *
from kuto.case import Case, Page
from kuto.running.runner import main
from kuto.utils.config import config
from kuto.utils.log import logger
from kuto.utils.pytest_util import depend, order, \
    data, file_data
from allure import *


__version__ = "0.0.62"
__description__ = "移动、web、接口自动化测试框架"
